#!/usr/bin/python
# -*- coding: utf-8 -*-
############################################################################
#    Copyright (C) 2007 by Romain Henriet                                  #
#    henriet.romain@libertysurf.fr                                         #
#                                                                          #
#    DesktopBasket is free software; you can redistribute it and/or modify #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    DesktopBasket is distributed in the hope that it will be useful,      #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

# This file contains Desktopbasket_config and Desktopbasket_actions classes

from kdecore import *
from commands import getoutput
import os, re
import karamba

# Desktopbasket_config contains the configuration parameters for DesktopBasket
class Desktopbasket_config:

	#########################################################################
	#                       Theme's version                                 #
	#                    should be updated for each release                 #
	#########################################################################
	version = "1.5alpha"

	authors = 'Romain Henriet (henriet.romain@libertysurf.fr)'
	contributors = 'Suslik (dd@accentsolution.com)'

	#_('white')
	# HTML colors
	colors = {'black':'000000000', 'white':'255255255', 'red':'255000000', 'blue':'000000255', 'yellow':'2552550', 'orange':'255165000', 'green':'000255000', 'purple':'160032240', 'maroon':'176048096', 'grey':'190190190', 'navy':'000000128', 'silver': '230230250'}

	def __init__(self, widget):

		print "Loading config"
		self.basketBase = self.searchForBasketBase()

		self.xborder=20
		self.yborder=12

		self.size=[0,0]
		
		self.smallNote = karamba.readConfigEntry(widget, "isReduced")=='True'

		self.showTags = karamba.readConfigEntry(widget, "showTags")!='False'
		
		self.desiredTag = karamba.readConfigEntry(widget, "desiredTag")
		if self.desiredTag:
			self.desiredTag=self.desiredTag.rsplit(';')

		# directory of the basket where to create quick notes
		self.quickNotesBasket = karamba.readConfigEntry(widget, "quickNotesBasket")

		self.titleType = karamba.readConfigEntry(widget, "titleType")
		if not self.titleType:
			self.titleType='Text'
			karamba.writeConfigEntry(widget, "titleType", self.titleType)
		if self.titleType=='Date':
			self.titleText = ""
		else:
			self.titleText = karamba.readConfigEntry(widget, "title")
			if not self.titleText:
				self.titleText = 'DesktopBasket'
				karamba.writeConfigEntry(widget, "title", self.titleText)
		
		self.size[0] = karamba.readConfigEntry(widget, "widgetWidth")
		if not self.size[0]:
			self.size[0]=350

		self.refreshRate = karamba.readConfigEntry(widget, "refreshRate")
		if not self.refreshRate:
			self.refreshRate=30

		self.basketNameColor = karamba.readConfigEntry(widget, "basketNameColor")
		if not self.basketNameColor:
			self.basketNameColor='Don\'t show'

		self.size[1] = karamba.readConfigEntry(widget, "noteHeight")
		if not self.size[1]:
			self.size[1]=200
			karamba.writeConfigEntry(widget, "noteHeight", str(self.size[1]))
			
		self.defaultColor = karamba.readConfigEntry(widget, "defaultColor")
		if not self.defaultColor:
			self.defaultColor = 'black'
			karamba.writeConfigEntry(widget, "defaultColor", self.defaultColor)
		else:
			try:
				self.titleText=unicode(self.titleText, 'utf8')
			except:
				try:
					self.titleText=unicode(self.titleText)
				except:
					pass

		self.resourcesPath = os.environ['HOME']+'/.superkaramba/desktopbasket'
		
		try:
			# i think open(blah).readline() closes the file auto, just like .read()
			# python is generally good at garbadge collection.
			self.installedVersion = open(self.resourcesPath + '/version').readline()
			print "DesktopBasket version is", self.installedVersion
		except:
			self.installedVersion = '0'

		themePath = karamba.getThemePath(widget)

		if self.installedVersion != Desktopbasket_config.version:	# using a new version

			if not os.path.exists(self.resourcesPath):
				os.makedirs(self.resourcesPath)
			if not themePath.endswith(".skz"):	# not running from a skz archive
				os.spawnlp(os.P_WAIT, 'cp', 'cp', '-r', themePath+'/images', self.resourcesPath)
				os.spawnlp(os.P_WAIT, 'cp', 'cp', '-r', themePath+'/editor', self.resourcesPath)
				os.spawnlp(os.P_WAIT, 'cp', 'cp', '-r', themePath+'/doc', self.resourcesPath)
			else:
				# extract image folder
				print '\"Images\" folder will be extracted'
				os.spawnlp(os.P_WAIT, 'unzip', 'unzip', '-o', themePath, 'images/*', '-d', self.resourcesPath)
				# extract editor folder
				print '\"Editor\" folder will be extracted'
				os.spawnlp(os.P_WAIT, 'unzip', 'unzip', '-o', themePath, 'editor/*', '-d', self.resourcesPath)
				# extract help folder
				print '\"Help\" folder will be extracted'
				os.spawnlp(os.P_WAIT, 'unzip', 'unzip', '-o', themePath, 'doc/*', '-d', self.resourcesPath)
			f=open(self.resourcesPath+'/version', 'w')
			f.write(Desktopbasket_config.version)
			f.close()
			# move old version's highlight file if there is one
			oldHlFilePath = os.environ['HOME']+'/.superkaramba/dektopbasket_highlights'
			if os.path.exists(oldHlFilePath):
				os.spawnlp(os.P_WAIT, 'mv', 'mv', oldHlFilePath, self.hlFilePath())

			self.installedVersion = Desktopbasket_config.version

			# initialize language and documentation
			lang = karamba.userLanguages(widget)
			langFile = None
			self.localeFile = ""
			docPath = self.resourcesPath+'/doc/en/index.docbook'
			for l in lang:
				fileContents = karamba.readThemeFile(widget, 'locales/'+l+'.mo')
				if fileContents.__len__() > 0:
					self.localeFile = l+'.mo'
					print 'Recherche de', self.resourcesPath+'/doc/'+l+'/index.docbook'
					if os.path.exists(self.resourcesPath+'/doc/'+l+'/index.docbook'):
						docPath = self.resourcesPath+'/doc/'+l+'/index.docbook'
					break
			karamba.writeConfigEntry(widget, "locale", self.localeFile)
			os.chdir(self.resourcesPath+'/doc')
			os.spawnlp(os.P_WAIT, 'meinproc', 'meinproc', docPath)

		self.localeFile = karamba.readConfigEntry(widget, "locale")

		self.bgImgFile = karamba.readConfigEntry(widget, "bgImg")
		if not self.bgImgFile:
			self.bgImgFile = self.imagesPath()+'/background.png'
			karamba.writeConfigEntry(widget, "bgImg", self.bgImgFile)
			
		self.ttBarImgFile = karamba.readConfigEntry(widget, "titleBar")
		if not self.ttBarImgFile:
			self.ttBarImgFile = self.imagesPath()+'/TitleBar.png'
			karamba.writeConfigEntry(widget, "titleBar", self.ttBarImgFile)

		print "Config Loaded"

	def helpPath(self):
		return self.resourcesPath+'/doc/index.html'

	def editorPath(self):
		return self.resourcesPath+'/editor'

	def imagesPath(self):
		return self.resourcesPath+'/images'

	def hlFilePath(self):
		return self.resourcesPath+'/highlights'

	def searchForBasketBase(self):
		# a small rewrite. This takes care of more potential errors
		basketBase = ''
		a = getoutput('kde-config --localprefix')
		if a and not a.count('kde-config'): # if there was an error of any sort, that string will be in the responce.
			try:
				f = open(a+'share/config/basketrc').read() # this way you don't need to close it. does it by itself after read
				basketBase = re.search('(?<=dataFolder=).+',f).group().replace('$HOME',os.environ['HOME']) + '/'
				# this is a bit long, but if any part above fails, we don't need anyof this.
				# thus, I put it all under one try
			except:
				# I bet default basket install puts something there... so, something better than "return" with empty hands
				basketBase = a + 'share/apps/basket/'
				# we can also make it a '' and check for it later.
			print 'Basket baskets folder :', basketBase
		# i hope you do not mind my switching to return basketBase.
		# it makes it easier to reuse this function later somewhere
		return basketBase

	def saveDesiredTag(self, widget):
		# writes the list of tags that select notes to display into the config file
		selString = ''
		for t in self.desiredTag:
			selString = selString+t+';'		# separates tags with ';'
		selString = selString[:-1]
		karamba.writeConfigEntry(widget, "desiredTag", selString)
	
class DisplayElements:

	def __init__(self, widget, config, _):

		self.widget= widget
		self.config = config
		
		self.iconLoader = KGlobal.iconLoader()

		karamba.resizeWidget(widget, config.size[0], config.size[1])
		self.bgImg = karamba.createBackgroundImage(widget, 0, 20, config.bgImgFile)
		karamba.resizeImage(widget, self.bgImg, config.size[0],config.size[1]-20)
		
		self.ttBarImg = karamba.createBackgroundImage(widget, 0, 0, config.ttBarImgFile)
		karamba.resizeImage(widget, self.ttBarImg, config.size[0],20)
		
		self.moveLeftButton = karamba.createImage(widget, config.xborder, 2, str(self.iconLoader.iconPath('1leftarrow', KIcon.Small, False)))
		karamba.attachClickArea(widget, self.moveLeftButton)
		
		self.moveRightButton = karamba.createImage(widget, 18+config.xborder, 2, str(self.iconLoader.iconPath('1rightarrow', KIcon.Small, False)))
		karamba.attachClickArea(widget, self.moveRightButton)
		
		self.basketButton = karamba.createImage(widget, config.size[0]-16-config.xborder, 2, str(self.iconLoader.iconPath('basket', KIcon.Small, False)))
		karamba.attachClickArea(widget, self.basketButton)
		
		self.cfgButton = karamba.createImage(widget, config.size[0]-36-config.xborder, 2, str(self.iconLoader.iconPath('configure', KIcon.Small, False)))
		karamba.attachClickArea(widget, self.cfgButton)
		
		self.title = karamba.createText(widget, config.xborder+40, -1, config.size[0]-80-2*config.xborder, 20, config.titleText)
		karamba.attachClickArea(widget, self.title)
		karamba.changeTextColor(widget, self.title, 0, 0, 0)
		karamba.setTextAlign(widget, self.title, 'CENTER')
		karamba.changeTextSize(widget, self.title, 14)

		# Menu to use Basket
		self.basketOptions = {}
		self.basketMenu = karamba.createMenu(widget)
		self.basketOptions['createNote'] = karamba.addMenuItem(widget, self.basketMenu,_("Create Note"), 'filenew')
		karamba.addMenuSeparator(widget, self.basketMenu)
		self.basketOptions['runBasket'] = karamba.addMenuItem(widget, self.basketMenu,_("Run Basket"), 'basket')

		# Menu to configure DesktopBasket
		self.configOptions = {}
		self.configMenu = karamba.createMenu(widget)
		self.configOptions['refresh'] = karamba.addMenuItem(widget, self.configMenu,_("Refresh"), 'reload')
		self.configOptions['refreshRate'] = karamba.addMenuItem(widget, self.configMenu,_("Change refresh rate"), 'reload')
		karamba.addMenuSeparator(widget, self.configMenu)
		self.configOptions['selecttag'] = karamba.addMenuItem(widget, self.configMenu,_("Choose tags"), 'find')
		self.configOptions['createDesktopTag'] = karamba.addMenuItem(widget, self.configMenu,_("Create and use the \"Desktop\" tag"), 'desktop')
		self.configOptions['quickBasket'] = karamba.addMenuItem(widget, self.configMenu,_("Select quick notes basket"), 'knotes')
		karamba.addMenuSeparator(widget, self.configMenu)
		self.configOptions['titleDate'] = karamba.addMenuItem(widget, self.configMenu,_("Show/Hide hour in title bar"), 'history')
		self.configOptions['title'] = karamba.addMenuItem(widget, self.configMenu,_("Change title"), 'text')
		self.configOptions['color'] = karamba.addMenuItem(widget, self.configMenu,_("Change default text color"), 'colorize')
		self.configOptions['showTags'] = karamba.addMenuItem(widget, self.configMenu,_("Show/Hide tags"), 'idea')
		self.configOptions['basketNameColor'] = karamba.addMenuItem(widget, self.configMenu,_("Change basket name color"), 'colorize')
		self.configOptions['size'] = karamba.addMenuItem(widget, self.configMenu,_("Change note size"), 'crop')
		self.configOptions['titleBar'] = karamba.addMenuItem(widget, self.configMenu,_("Change title bar"), 'frame_image')
		self.configOptions['background'] = karamba.addMenuItem(widget, self.configMenu,_("Change background"), 'frame_image')
		self.configOptions['resetTheme'] = karamba.addMenuItem(widget, self.configMenu,_("Reset theme"), 'remove')
		karamba.addMenuSeparator(widget, self.configMenu)
		self.configOptions['help'] =  karamba.addMenuItem(widget, self.configMenu,_("Help"), 'help')
		self.configOptions['about'] = karamba.addMenuItem(widget, self.configMenu,_("About"), 'superkaramba')
		

		self.highlightOptions={}
		self.highlightMenu = karamba.createMenu(widget)
		a = os.listdir(config.imagesPath()+'/highlight')
		self.highlightOptions["None"] = karamba.addMenuItem(widget, self.highlightMenu, '', config.imagesPath()+'/highlight/None')
		for i in a:
			if i.endswith('.png'):
				self.highlightOptions[i] = karamba.addMenuItem(widget, self.highlightMenu, '', config.imagesPath()+'/highlight/'+i)

		self.highlightIcon = str(self.iconLoader.iconPath('bookmark', KIcon.Small, False))

		self.dropArea = karamba.acceptDrops(widget)

# Desktopbasket_actions contains actions's id to identify clicked meters
class Desktopbasket_actions:

	def __init__(self):
		self.select_img_pid = None
		self.select_title_pid = None
		self.select_size_pid = None
		self.select_color_pid = None
		self.select_ttBar_pid = None
		self.select_tag_pid = None
		self.select_refresh_pid = None
		self.select_basketnamecolor_pid = None
		self.create_note_pid = None	# after the basket has been chosen
		self.importFile_pid = None
		self.select_quickBasket_pid = None
		